/*
 * The contents of this file are subject to the terms
 * of the Common Development and Distribution License
 * (the "License").  You may not use this file except
 * in compliance with the License.
 *
 * You can obtain a copy of the license at
 * glassfish/bootstrap/legal/CDDLv1.0.txt or
 * https://glassfish.dev.java.net/public/CDDLv1.0.html.
 * See the License for the specific language governing
 * permissions and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL
 * HEADER in each file and include the License file at
 * glassfish/bootstrap/legal/CDDLv1.0.txt.  If applicable,
 * add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your
 * own identifying information: Portions Copyright [yyyy]
 * [name of copyright owner]
 */

package com.sun.enterprise.web.connector.grizzly.cometd;

import com.sun.enterprise.web.connector.grizzly.comet.CometContext;
import com.sun.enterprise.web.connector.grizzly.comet.CometEvent;
import com.sun.enterprise.web.connector.grizzly.cometd.bayeux.Connect;
import com.sun.enterprise.web.connector.grizzly.cometd.bayeux.Data;
import com.sun.enterprise.web.connector.grizzly.cometd.bayeux.Disconnect;
import com.sun.enterprise.web.connector.grizzly.cometd.bayeux.Handshake;
import com.sun.enterprise.web.connector.grizzly.cometd.bayeux.Reconnect;
import com.sun.enterprise.web.connector.grizzly.cometd.bayeux.Subscribe;
import com.sun.enterprise.web.connector.grizzly.cometd.bayeux.Unsubscribe;
import com.sun.enterprise.web.connector.grizzly.cometd.bayeux.Verb;
import java.io.IOException;
import java.util.Random;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentLinkedQueue;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


public class BayeuxCometHandler extends BayeuxCometHandlerBase{
    
    public final static String BAYEUX_COMET_HANDLER = "bayeuxCometHandler";

    private ConcurrentLinkedQueue<String> clientIds = new ConcurrentLinkedQueue<String>();
    
    private Random random = new Random();
    
    private ConcurrentHashMap<String,CometContext> channels =
            new ConcurrentHashMap<String,CometContext>();
    
    
    public void onHandshake(CometEvent event) throws IOException {            
        CometdContext cometdContext = (CometdContext)event.attachment();
        
        HttpServletRequest req = (HttpServletRequest)
                cometdContext.getRequest();
        HttpServletResponse res = (HttpServletResponse)
                cometdContext.getResponse();
        Handshake handshake = (Handshake)cometdContext.getVerb();
        
        boolean handshakeOK = true;
        String clientId  = "";
        synchronized(random){
            clientId = String.valueOf(Long.toHexString(random.nextLong()));
        }
        clientIds.add(clientId);
        
        // check....
        if (handshakeOK){
            handshake.setClientId(clientId);
        }
        
        System.out.println("response: " + handshake.toJSON());
        
        res.setContentType("text/plain; charset=utf-8");            
        res.getWriter().write(handshake.toJSON());
        res.getWriter().flush();
    }
    
    
    public void onConnect(CometEvent event) throws IOException {
        CometdContext cometdContext = (CometdContext)event.attachment();
        
        HttpServletRequest req = (HttpServletRequest)
                cometdContext.getRequest();
        HttpServletResponse res = (HttpServletResponse)
                cometdContext.getResponse();
        Connect connect = (Connect)cometdContext.getVerb();            
        System.out.println("response: " + connect.toJSON());
        
        res.setContentType("text/plain; charset=utf-8");            
        res.getWriter().write(connect.toJSON());
        res.getWriter().flush();
    }
    
    
    public void onDisconnect(CometEvent event) throws IOException {
        CometdContext cometdContext = (CometdContext)event.attachment();
        
        HttpServletRequest req = (HttpServletRequest)
                cometdContext.getRequest();
        HttpServletResponse res = (HttpServletResponse)
                cometdContext.getResponse();
        Disconnect disconnect = (Disconnect)cometdContext.getVerb();            
        System.out.println("response: " + disconnect.toJSON());
        
        res.setContentType("text/plain; charset=utf-8");            
        res.getWriter().write(disconnect.toJSON());
        res.getWriter().flush();
    }
    

    public void onReconnect(CometEvent event) throws IOException {
        CometdContext cometdContext = (CometdContext)event.attachment();
        
        HttpServletRequest req = (HttpServletRequest)
                cometdContext.getRequest();
        HttpServletResponse res = (HttpServletResponse)
                cometdContext.getResponse();
        Reconnect reconnect = (Reconnect)cometdContext.getVerb();                    
        System.out.println("response: " + reconnect.toJSON());
                       
        CometContext cometContext = event.getCometContext();
        DataHandler requestHandler = new DataHandler();
        requestHandler.attach(new Object[]{req,res});
        cometContext.addCometHandler(requestHandler);   
        
        res.setContentType("text/plain; charset=utf-8");            
        res.getOutputStream().println(reconnect.toJSON());
        res.getOutputStream().flush();
    }
    
    
    public void onSubscribe(CometEvent event) throws IOException {
        CometdContext cometdContext = (CometdContext)event.attachment();
        
        HttpServletRequest req = (HttpServletRequest)
                cometdContext.getRequest();
        HttpServletResponse res = (HttpServletResponse)
                cometdContext.getResponse();
        Subscribe subscribe = (Subscribe)cometdContext.getVerb();   

        System.out.println("response: " + subscribe.toJSON());
         
        res.setContentType("text/plain; charset=utf-8");            
        res.getWriter().write(subscribe.toJSON());
        res.getWriter().flush();
    }
    
    
    public void onUnsubscribe(CometEvent event) throws IOException {
        CometdContext cometdContext = (CometdContext)event.attachment();
        
        HttpServletRequest req = (HttpServletRequest)
                cometdContext.getRequest();
        HttpServletResponse res = (HttpServletResponse)
                cometdContext.getResponse();
        Unsubscribe unsubscribe = (Unsubscribe)cometdContext.getVerb();   

        System.out.println("response: " + unsubscribe.toJSON());
         
        res.setContentType("text/plain; charset=utf-8");            
        res.getWriter().write(unsubscribe.toJSON());
        res.getWriter().flush();
    }
    
    
    public void onData(CometEvent event) throws IOException {
        System.out.println("=========onData");
        CometdContext cometdContext = (CometdContext)event.attachment();
        
        HttpServletRequest req = (HttpServletRequest)
                cometdContext.getRequest();
        HttpServletResponse res = (HttpServletResponse)
                cometdContext.getResponse();
        Data data = (Data)cometdContext.getVerb();           
        
        System.out.println("response: " + data.toJSON());      
         
        res.setContentType("text/plain; charset=utf-8");            
        res.getWriter().write(data.toJSON());
        res.getWriter().flush();       
        event.getCometContext().notify(data);
    }

    
    public void onPing(CometEvent event) throws IOException {
    }
    
    
    public void onStatus(CometEvent event) throws IOException {
    }
    
    
    
    public void attach(Object attachment) {
    }
    
    public void onInitialize(CometEvent event) throws IOException {
    }
    
    public void onTerminate(CometEvent event) throws IOException {
    }
    
    public void onInterrupt(CometEvent event) throws IOException {
    }
    
    
    public final static CometdContext newCometdContext(final HttpServletRequest req, 
            final HttpServletResponse res,final Verb verb){
        return new CometdContext<HttpServletRequest,HttpServletResponse>(){
            
            public Verb getVerb(){
                return verb;
            }
            
            public HttpServletRequest getRequest(){
                return req;
            }
                      
            public HttpServletResponse getResponse(){
                return res;
            }            
        };
    }

    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
}