/*
 * The contents of this file are subject to the terms
 * of the Common Development and Distribution License
 * (the License).  You may not use this file except in
 * compliance with the License.
 *
 * You can obtain a copy of the license at
 * https://glassfish.dev.java.net/public/CDDLv1.0.html or
 * glassfish/bootstrap/legal/CDDLv1.0.txt.
 * See the License for the specific language governing
 * permissions and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL
 * Header Notice in each file and include the License file
 * at glassfish/bootstrap/legal/CDDLv1.0.txt.
 * If applicable, add the following below the CDDL Header,
 * with the fields enclosed by brackets [] replaced by
 * you own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
 */

package com.sun.enterprise.web.connector.grizzly.cometd;

import com.sun.enterprise.web.connector.grizzly.comet.*;
import com.sun.enterprise.web.connector.grizzly.cometd.bayeux.Data;
import com.sun.enterprise.web.connector.grizzly.cometd.bayeux.Verb;
import com.sun.enterprise.web.connector.grizzly.cometd.bayeux.VerbUtils;
import java.io.IOException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 *
 * @author Jeanfrancois Arcand
 */
public class ServletEventRouter implements 
        EventRouter<HttpServletRequest,HttpServletResponse>{
       
    private final static String JSON_MESSAGES = "message";
    
    private CometContext cometContext;
        
    public ServletEventRouter(CometContext cometContext) {
        this.setCometContext(cometContext);
    }

    
    public void coordinate(final HttpServletRequest req, 
            final HttpServletResponse res) throws IOException {

        String[] messages = req.getParameterValues(JSON_MESSAGES);        
        if (messages != null && messages.length > 0){
            for(String message: messages){
                System.out.println("message: " + message);
                final Verb verb = VerbUtils.parse(JSONParser.parse(message));
                
                // Notify our listener;
                getCometContext().notify(
                        BayeuxCometHandler.newCometdContext(req,res,verb),
                        CometEvent.NOTIFY,
                        (Integer)cometContext.getAttribute(
                            BayeuxCometHandler.BAYEUX_COMET_HANDLER));             
            }
        } else {
            //throw new IOException("Invalid Bayeux request");
        }
    }  

    public CometContext getCometContext() {
        return cometContext;
    }

    
    public void setCometContext(CometContext cometContext) {
        this.cometContext = cometContext;
    }
    
}
