/*
 * The contents of this file are subject to the terms
 * of the Common Development and Distribution License
 * (the License).  You may not use this file except in
 * compliance with the License.
 *
 * You can obtain a copy of the license at
 * https://glassfish.dev.java.net/public/CDDLv1.0.html or
 * glassfish/bootstrap/legal/CDDLv1.0.txt.
 * See the License for the specific language governing
 * permissions and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL
 * Header Notice in each file and include the License file
 * at glassfish/bootstrap/legal/CDDLv1.0.txt.
 * If applicable, add the following below the CDDL Header,
 * with the fields enclosed by brackets [] replaced by
 * you own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
 */

package com.sun.enterprise.web.connector.grizzly.cometd.bayeux;

import java.lang.reflect.Array;
import java.util.HashMap;
import java.util.Map;

/**
 *
 * @author Jeanfrancois Arcand
 */
public class VerbUtils {
    
    private final static String META = "/meta";
    private final static String HANDSHAKE = "/handshake";
    private final static String CONNECT = "/connect";
    private final static String DISCONNECT = "/disconnect";    
    private final static String RECONNECT = "/reconnect";    
    private final static String SUBSCRIBE = "/subscribe";
    private final static String UNSUBSCRIBE = "/unsubscribe";    
    private final static String STATUS = "/status";
    private final static String PING = "/ping";
    private final static String DATA = "/data";    
    
    public VerbUtils() {
    }
    
    
    public static Verb parse(Object verb){   
        
        if (verb.getClass().isArray()){
            int length = Array.getLength(verb);
            for (int i=0; i < length; i++){
                return parseMap((Map)Array.get(verb,i));
            }
        }
        throw new RuntimeException("Wrong type");  
    }
    
    
    protected static Verb parseMap(Map map){
        
        String channel = (String)map.get("channel");
        
        if (!channel.startsWith(META)){
            return newData(map);
        }
        
        if (channel.indexOf(HANDSHAKE) != -1){
            return newHandshake(map);
        } else if (channel.indexOf(CONNECT) != -1){
            return newConnect(map);
        } else if (channel.indexOf(DISCONNECT) != -1){
            return newDisconnect(map);            
        } else if (channel.indexOf(RECONNECT) != -1){
            return newReconnect(map);
        } else if (channel.indexOf(SUBSCRIBE) != -1){
            return newSubscribe(map);
        } else if (channel.indexOf(UNSUBSCRIBE) != -1){
            return newUnsubscribe(map);
        } else if (channel.indexOf(PING) != -1){
            return newPing(map);
        } else if (channel.indexOf(STATUS) != -1){
            return newStatus(map);
        }        
        return null;
    }
    
    
    private final static Handshake newHandshake(Map map){
        Handshake handshake = new Handshake();
        
        handshake.setAuthScheme((String)map.get("authScheme"));
        handshake.setAuthUser((String)map.get("authUser"));
        handshake.setAuthToken((String)map.get("authToken"));
        handshake.setChannel((String)map.get("channel"));
        handshake.setVersion(String.valueOf((Double)map.get("version")));
        handshake.setMinimumVersion(String.valueOf((Double)map.get("minimumVersion")));
        //handshake.setSupportedConnectionTypes(map.get("supportedConnectionTypes"));
        handshake.setAdvice(new Advice());
        return handshake; 
    }
    
    
    private final static Connect newConnect(Map map){
        Connect connect = new Connect();
        
        connect.setAuthToken((String)map.get("authToken"));
        connect.setChannel((String)map.get("channel"));
        connect.setClientId((String)map.get("clientId"));
        connect.setConnectionType((String)map.get("connectionType"));
        
        return connect;
    }
    
    
    private final static Disconnect newDisconnect(Map map){
        Disconnect disconnect = new Disconnect();
        
        disconnect.setAuthToken((String)map.get("authToken"));
        disconnect.setChannel((String)map.get("channel"));
        disconnect.setClientId((String)map.get("clientId"));
        disconnect.setConnectionType((String)map.get("connectionType"));
        
        return disconnect;
    }    
    
    
    private final static Reconnect newReconnect(Map map){
        Reconnect reconnect = new Reconnect();
        
        reconnect.setAuthToken((String)map.get("authToken"));
        reconnect.setChannel((String)map.get("channel"));
        reconnect.setClientId((String)map.get("clientId"));
        reconnect.setConnectionType((String)map.get("connectionType"));
        
        return reconnect;
    } 
    
    
    private final static Data newData(Map map){
        Data data = new Data();
        
        data.setChannel((String)map.get("channel"));
        data.setClientId((String)map.get("clientId"));
        data.setConnectionId((String)map.get("connectionId"));
        data.setData((HashMap)map.get("data"));
        
        return data;
    }
    
    
    private final static Subscribe newSubscribe(Map map){
        Subscribe subscribe = new Subscribe();
        
        subscribe.setChannel((String)map.get("channel"));
        subscribe.setAuthToken((String)map.get("authToken"));
        subscribe.setSubscription((String)map.get("subscription"));
        
        return subscribe;
    }
    
        
    private final static Unsubscribe newUnsubscribe(Map map){
        Unsubscribe unsubscribe = new Unsubscribe();
        
        unsubscribe.setChannel((String)map.get("channel"));
        unsubscribe.setAuthToken((String)map.get("authToken"));
        unsubscribe.setSubscription((String)map.get("subscription"));
        
        return unsubscribe;
    }
    
    
    private final static Ping newPing(Map map){
        Ping ping = new Ping();
        
        ping.setChannel((String)map.get("channel"));
        return ping;
    }
    
    
    private final static Status newStatus(Map map){
        Status status = new Status();
        
        status.setChannel((String)map.get("channel"));
        return status;        
    }
    
}
