/*
 * The contents of this file are subject to the terms
 * of the Common Development and Distribution License
 * (the "License").  You may not use this file except
 * in compliance with the License.
 *
 * You can obtain a copy of the license at
 * glassfish/bootstrap/legal/CDDLv1.0.txt or
 * https://glassfish.dev.java.net/public/CDDLv1.0.html.
 * See the License for the specific language governing
 * permissions and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL
 * HEADER in each file and include the License file at
 * glassfish/bootstrap/legal/CDDLv1.0.txt.  If applicable,
 * add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your
 * own identifying information: Portions Copyright [yyyy]
 * [name of copyright owner]
 */
package com.sun.enterprise.web.connector.grizzly.cometd.servlet;


import com.sun.enterprise.web.connector.grizzly.comet.CometContext;
import com.sun.enterprise.web.connector.grizzly.comet.CometEngine;
import com.sun.enterprise.web.connector.grizzly.comet.CometEvent;
import com.sun.enterprise.web.connector.grizzly.cometd.*;
import com.sun.enterprise.web.connector.grizzly.cometd.bayeux.Connect;
import com.sun.enterprise.web.connector.grizzly.cometd.bayeux.Handshake;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Random;
import java.util.concurrent.ConcurrentLinkedQueue;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


/**
 * Simple CometChat.
 *
 * @author Jeanfrancois Arcand
 */
public class CometServlet extends HttpServlet{
    
    private String contextPath;
    
    private BayeuxCometHandler bayeuxCometHandler;
    
    private EventRouter eventRouter;
    
    private boolean initialized = false;
    
    public CometServlet() {
    }
    
    public void init(ServletConfig config) throws ServletException {
        super.init(config);
        contextPath = config.getServletContext().getContextPath() + "/cometd";
        CometEngine cometEngine = CometEngine.getEngine();
        CometContext context = cometEngine.register(contextPath);
        context.setExpirationDelay(60 * 60 * 1000);
        
        bayeuxCometHandler = new BayeuxCometHandler();
        eventRouter = new ServletEventRouter(context);     
    }
    
    
    public void doGet(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException {
        doPost(request,response);
    }
    
    
    public void doPost(HttpServletRequest hreq, HttpServletResponse hres)
        throws ServletException, IOException {
        
        CometEngine cometEngine = CometEngine.getEngine();
        CometContext cometContext = cometEngine.getCometContext(contextPath);
        
        // XXX Pool instance
        CometdRequest cometdReq = new CometdRequest<HttpServletRequest>(hreq){            
            public String[] getParameterValues(String s) {
                return request.getParameterValues(s);
            }
        };
        
        CometdResponse cometdRes = new CometdResponse<HttpServletResponse>(hres){            
            public void write(String s) throws IOException{
                response.getWriter().write(s);
            }

            public void flush() throws IOException{
                response.getWriter().flush();
            }

            public void setContentType(String s) {
                response.setContentType(s);
            }
        };        
        
        if (!initialized){
            synchronized(cometContext){
                if (!initialized){
                     int mainHandlerHash = 
                            cometContext.addCometHandler(bayeuxCometHandler,true);
                    cometContext.addAttribute(BayeuxCometHandler.BAYEUX_COMET_HANDLER,
                                              mainHandlerHash);
                    initialized = true;
                }
            }
        }
        
        eventRouter.route(cometdReq,cometdRes);
    }
    
    // --------------------------------------------------------- Async Hook ---/

}
