
/*
 * The contents of this file are subject to the terms
 * of the Common Development and Distribution License
 * (the License).  You may not use this file except in
 * compliance with the License.
 *
 * You can obtain a copy of the license at
 * https://glassfish.dev.java.net/public/CDDLv1.0.html or
 * glassfish/bootstrap/legal/CDDLv1.0.txt.
 * See the License for the specific language governing
 * permissions and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL
 * Header Notice in each file and include the License file
 * at glassfish/bootstrap/legal/CDDLv1.0.txt.
 * If applicable, add the following below the CDDL Header,
 * with the fields enclosed by brackets [] replaced by
 * you own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
 *
 * Portions Copyright Apache Software Foundation.
 */

package com.sun.enterprise.web.connector.grizzly.cometd.standalone;


import com.sun.enterprise.web.connector.grizzly.Constants;
import com.sun.enterprise.web.connector.grizzly.FileCache;
import com.sun.enterprise.web.connector.grizzly.FileCacheFactory;
import com.sun.enterprise.web.connector.grizzly.SelectorThread;
import com.sun.enterprise.web.connector.grizzly.SocketChannelOutputBuffer;
import com.sun.enterprise.web.connector.grizzly.comet.CometContext;
import com.sun.enterprise.web.connector.grizzly.comet.CometEngine;
import com.sun.enterprise.web.connector.grizzly.cometd.BayeuxCometHandler;
import com.sun.enterprise.web.connector.grizzly.cometd.CometdRequest;
import com.sun.enterprise.web.connector.grizzly.cometd.CometdResponse;
import com.sun.enterprise.web.connector.grizzly.cometd.EventRouter;
import com.sun.enterprise.web.connector.grizzly.standalone.StaticResourceServer;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.nio.channels.SocketChannel;
import java.util.Enumeration;
import java.util.Properties;
import java.util.logging.Level;
import org.apache.coyote.ActionCode;

import org.apache.coyote.Adapter;
import org.apache.coyote.Request;
import org.apache.coyote.Response;
import org.apache.coyote.tomcat5.CoyoteConnector;
import org.apache.tomcat.util.buf.Ascii;
import org.apache.tomcat.util.buf.ByteChunk;
import org.apache.tomcat.util.buf.MessageBytes;
import org.apache.tomcat.util.http.MimeHeaders;
import org.apache.tomcat.util.http.Parameters;

/**
 * Standalone Cometd implementation.
 *
 * @author Jeanfrancois Arcand
 */
public class CometdAdapter implements Adapter {
    
    private String contextPath;
    
    private BayeuxCometHandler bayeuxCometHandler;
    
    private EventRouter eventRouter;
    
    private boolean initialized = false;
    
    public static final int ADAPTER_NOTES = 1;
    
    public CometdAdapter() {
        this.contextPath = "/cometd/cometd";
        CometEngine cometEngine = CometEngine.getEngine();
        CometContext cometContext = cometEngine.register(contextPath);
        cometContext.setExpirationDelay(60 * 60 * 1000);
        
        bayeuxCometHandler = new BayeuxCometHandler();
        eventRouter = new StandaloneEventRouter(cometContext);        
    }
    
    
    public void service(Request req, final Response res) throws Exception {
        MessageBytes mb = req.requestURI();
        ByteChunk requestURI = mb.getByteChunk();
        String uri = req.requestURI().toString();

        if (!uri.endsWith(contextPath)){
            serveFile(req,res);
            return;
        }
        
        
        CometEngine cometEngine = CometEngine.getEngine();
        CometContext cometContext = cometEngine.getCometContext(contextPath);        
        if (!initialized){
            synchronized(cometContext){
                if (!initialized){
                     int mainHandlerHash = 
                            cometContext.addCometHandler(bayeuxCometHandler,true);
                    cometContext.addAttribute(BayeuxCometHandler.BAYEUX_COMET_HANDLER,
                                              mainHandlerHash);
                    initialized = true;
                }
            }
        }
        
        CometdRequest cometdReq = (CometdRequest) req.getNote(ADAPTER_NOTES);
        CometdResponse cometdRes = (CometdResponse) res.getNote(ADAPTER_NOTES);
        res.sendHeaders();
                    
        if (cometdReq == null) {
            cometdReq = new CometdRequest<Request>(req) {
                
                private boolean requestParametersParsed = false;
                protected byte[] postData = null;
                private ByteChunk chunk = new ByteChunk();
                private byte[] body = new byte[8192];
                
                public String[] getParameterValues(String s) {
                    
                    Parameters parameters = request.getParameters();
                    requestParametersParsed = true;
                    
                    parameters.setEncoding
                            (org.apache.coyote.Constants.DEFAULT_CHARACTER_ENCODING);
                    parameters.setQueryStringEncoding
                            (org.apache.coyote.Constants.DEFAULT_CHARACTER_ENCODING);
                    parameters.handleQueryParameters();
                    int len = request.getContentLength();
                     
                    if (len > 0) {
                        try {
                            
                            byte[] formData = getPostBody();
                            if (formData != null) {
                                parameters.processParameters(formData, 0, len);
                            }
                            // END SJSAS 6346738
                        } catch (Throwable t) {
                            ; // Ignore
                        }
                    }
                    
                    return parameters.getParameterValues(s);
                }

                protected byte[] getPostBody() throws IOException {
                    
                    int len = request.getContentLength(); 
                    
                    int actualLen = readPostBody(chunk, len);
                    if (actualLen == len) {                       
                        chunk.substract(body,0,chunk.getLength());
                        chunk.recycle();
                        return body;
                    }
                    
                    return null;
                }
                
                
                /**
                 * Read post body in an array.
                 */
                protected int readPostBody(ByteChunk chunk,int len) throws IOException {
                    
                    int offset = 0;
                    do {
                        int inputLen = request.doRead(chunk);
                        if (inputLen <= 0) {
                            return offset;
                        }
                        offset += inputLen;
                    } while ((len - offset) > 0);
                    return len;                   
                }
                
            };
            
            cometdRes = new CometdResponse<Response>(res) {
                public void write(String s) throws IOException {
                    ((SocketChannelOutputBuffer)res.getOutputBuffer())
                    .realWriteBytes(s.getBytes(),0,s.length());
                    flush();
                }
                
                public void flush() throws IOException {
                    ((SocketChannelOutputBuffer)res.getOutputBuffer()).flush();
                }
                
                public void setContentType(String s) {
                    response.setContentType(s);
                }
            };
            
            // Set as notes
            req.setNote(ADAPTER_NOTES, cometdReq);
            res.setNote(ADAPTER_NOTES, cometdRes);
        }  else {
            cometdReq.setRequest(req);
            cometdRes.setResponse(res);
        }
        eventRouter.route(cometdReq,cometdRes);
    }
    
    
    public void fireAdapterEvent(String string, Object object) {
    }
    
    public void afterService(Request req, Response res) throws Exception {
        try{
            req.action( ActionCode.ACTION_POST_REQUEST , null);
        }catch (Throwable t) {
            t.printStackTrace();
        } finally {
            // Recycle the wrapper request and response
            req.recycle();
            res.recycle();
        }
    }
    
    
    static Properties contentTypes=new Properties();
    
    static {
        initContentTypes();
    }
    
    static void initContentTypes() {
        contentTypes.put("html", "text/html");
        contentTypes.put("txt", "text/plain");
    }
    
    private String rootFolder;
    
    private File rootFolderF;
    
    /**
     * Grizzly FileCache
     */
    private FileCache fileCache;
    
    public void setRootFolder(String s) {
        rootFolder = s;
        rootFolderF = new File(rootFolder);
        try {
            rootFolder = rootFolderF.getCanonicalPath();
            SelectorThread.logger().log(Level.INFO, "Servicing page from: "
                    + rootFolder);
            
        } catch (IOException e) {
        }
    }
    
    
    private boolean serveFile(Request req, final Response res) throws Exception {
        MessageBytes mb = req.requestURI();
        ByteChunk requestURI = mb.getByteChunk();
        
        if (fileCache == null){
            fileCache = FileCacheFactory.getFactory(req.getLocalPort())
            .getFileCache();
        }
        
        if (!fileCache.sendCache(requestURI.getBytes(), requestURI.getStart(),
                requestURI.getLength(), getChannel(res),
                keepAlive(req))){
            
            String uri = req.requestURI().toString();
            if (uri.indexOf("..") >= 0) {
                res.setStatus(404);
                return false;
            }
            
            File file = new File(rootFolderF, uri);
            if (!file.getCanonicalPath().startsWith(rootFolder)) {
                res.setStatus(404);
                return false;
            }
            
            if (file.isDirectory()) {
                file = new File(file, "index.html");
            }
            
            if (!file.exists()) {
                SelectorThread.logger().log(Level.INFO,"File not found  " + file);
                res.setStatus(404);
                return false;
            }
            res.setStatus(200);
            
            int dot=uri.lastIndexOf(".");
            if( dot > 0 ) {
                String ext=uri.substring(dot+1);
                String ct=getContentType(ext);
                if( ct!=null) {
                    res.setContentType(ct);
                }
            }
            
            res.setContentLength((int)file.length());
            res.sendHeaders();
            
            FileInputStream fis = new FileInputStream(file);
            
            byte b[] = new byte[4096];
            ByteChunk chunk = new ByteChunk();
            int rd = 0;
            while ((rd = fis.read(b)) > 0) {
                chunk.setBytes(b, 0, rd);
                res.doWrite(chunk);
            }
            
            fileCache.add(FileCache.DEFAULT_SERVLET_NAME,rootFolder,uri,
                    res.getMimeHeaders(),false);
        }
        
        return true;
    }
    
    
    private SocketChannel getChannel(Response res) {
        SocketChannelOutputBuffer buffer = (SocketChannelOutputBuffer) res.getOutputBuffer();
        SocketChannel channel = buffer.getChannel();
        return channel;
    }
    
    
    /**
     * Get the keep-alive header.
     */
    private boolean keepAlive(Request request){
        MimeHeaders headers = request.getMimeHeaders();
        
        // Check connection header
        MessageBytes connectionValueMB = headers.getValue("connection");
        if (connectionValueMB != null) {
            ByteChunk connectionValueBC = connectionValueMB.getByteChunk();
            if (findBytes(connectionValueBC, Constants.CLOSE_BYTES) != -1) {
                return false;
            } else if (findBytes(connectionValueBC,
                    Constants.KEEPALIVE_BYTES) != -1) {
                return true;
            }
        }
        return true;
    }
    
    
    /**
     * Specialized utility method: find a sequence of lower case bytes inside
     * a ByteChunk.
     */
    protected int findBytes(ByteChunk bc, byte[] b) {
        
        byte first = b[0];
        byte[] buff = bc.getBuffer();
        int start = bc.getStart();
        int end = bc.getEnd();
        
        // Look for first char
        int srcEnd = b.length;
        
        for (int i = start; i <= (end - srcEnd); i++) {
            if (Ascii.toLower(buff[i]) != first) continue;
            // found first char, now look for a match
            int myPos = i+1;
            for (int srcPos = 1; srcPos < srcEnd; ) {
                if (Ascii.toLower(buff[myPos++]) != b[srcPos++])
                    break;
                if (srcPos == srcEnd) return i - start; // found it
            }
        }
        return -1;
    }
    
    
    public String getContentType( String ext ) {
        return contentTypes.getProperty( ext, "text/plain" );
    }
    
    public String getRootFolder() {
        return rootFolder;
    }
}