
/*
 * The contents of this file are subject to the terms 
 * of the Common Development and Distribution License 
 * (the License).  You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the license at 
 * https://glassfish.dev.java.net/public/CDDLv1.0.html or
 * glassfish/bootstrap/legal/CDDLv1.0.txt.
 * See the License for the specific language governing 
 * permissions and limitations under the License.
 * 
 * When distributing Covered Code, include this CDDL 
 * Header Notice in each file and include the License file 
 * at glassfish/bootstrap/legal/CDDLv1.0.txt.  
 * If applicable, add the following below the CDDL Header, 
 * with the fields enclosed by brackets [] replaced by
 * you own identifying information: 
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
 */
package com.sun.enterprise.web.connector.grizzly.realtime;

import com.sun.enterprise.web.connector.grizzly.LinkedListPipeline;
import com.sun.enterprise.web.connector.grizzly.Task;
import javax.realtime.PriorityParameters;
import javax.realtime.PriorityScheduler;
import javax.realtime.RealtimeThread;

/**
 *
 * 
 */
public class RealTimePipeline extends LinkedListPipeline{
    
    /**
     * Pool of RealTime worker threads.
     */
    private RealTimeWorkerThread workerThreads[];
    
    
    // --------------------------------------------- Constructor -----------//
    
    public RealTimePipeline() {
         super();
    }
    
    
    public RealTimePipeline(int maxThreads, int minThreads, String name, 
                            int port, int priority){
                        
        this.maxThreads = maxThreads;
        this.port = port;
        this.name = name;
        this.minThreads = minThreads;
        this.priority = priority;
        
        if ( minThreads < minSpareThreads )
            minSpareThreads = minThreads;
        
    }

    
    public RealTimePipeline(int maxThreads, int minThreads, String name, 
                            int port){
                        
        this(maxThreads,minThreads,name,port,Thread.NORM_PRIORITY);
    }
    
    
    // ---------------------------------------- Thread Management -----------//
    
        
    /**
     * Init the <code>Pipeline</code> by initializing the required
     * <code>WorkerThread</code>. Default value is 10
     */
    public void initPipeline(){
        
        //System.out.print("Initializing RealTimePipeline");
        
        workerThreads = new RealTimeWorkerThread[maxThreads];  
        RealTimeWorkerThread workerThread;
        PriorityScheduler scheduler = PriorityScheduler.instance();
        PriorityParameters pp = 
                new PriorityParameters(scheduler.getMaxPriority());
        for (int i=0; i < maxThreads; i++){
            workerThread = new RealTimeWorkerThread(pp, this);
            workerThreads[i] = workerThread;
            threadCount++; 
        } 
   }

    
    /**
     * Start the <code>Pipeline</code> and all associated 
     * <code>WorkerThread</code>
     */
    public void startPipeline(){
        if (!isStarted) {
            for (int i=0; i < maxThreads; i++){
                workerThreads[i].start();
            }
            isStarted = true;
        }
    }
    

    /**
     * Stop the <code>Pipeline</code> and all associated
     * <code>WorkerThread</code>
     */
    public void stopPipeline(){
        if (isStarted) {
            for (int i=0; i < maxThreads; i++){
                workerThreads[i].terminate();
            }
            isStarted = false;
        }
    }
    
      
    
    /**
     * Add an object to this pipeline
     */
    public synchronized void addTask(Task task) {
        System.out.println("Executing RealTimeThread");
        super.addTask(task);        
    }
    
}
