/*
 * The contents of this file are subject to the terms
 * of the Common Development and Distribution License
 * (the License).  You may not use this file except in
 * compliance with the License.
 *
 * You can obtain a copy of the license at
 * https://glassfish.dev.java.net/public/CDDLv1.0.html or
 * glassfish/bootstrap/legal/CDDLv1.0.txt.
 * See the License for the specific language governing
 * permissions and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL
 * Header Notice in each file and include the License file
 * at glassfish/bootstrap/legal/CDDLv1.0.txt.
 * If applicable, add the following below the CDDL Header,
 * with the fields enclosed by brackets [] replaced by
 * you own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
 */

package com.sun.enterprise.web.connector.grizzly.realtime;

import com.sun.enterprise.web.connector.grizzly.SelectorThread;
import java.util.ArrayList;
import java.util.StringTokenizer;
import java.util.logging.Level;
import javax.realtime.PriorityParameters;
import javax.realtime.PriorityScheduler;
import javax.realtime.RealtimeThread;

/**
 *
 * @author Jeanfrancois Arcand
 */
public class RealTimeSelectorThread extends SelectorThread{
    
    public static String PRIVILEGED_PORTS = 
            "com.sun.enterprise.web.connector.grizzly.privilegedPorts";
    
    private static ArrayList<Integer> privilegedPorts =
            new ArrayList<Integer>();
    
    
    static{
        if ( System.getProperty(PRIVILEGED_PORTS) != null){
            StringTokenizer privList = 
                    new StringTokenizer(
                        System.getProperty(PRIVILEGED_PORTS),",");
            while (privList.hasMoreElements()){
                privilegedPorts.add(Integer.parseInt(privList.nextToken()));
            }
        }   
    }     
    
    
    public RealTimeSelectorThread() {
    }
    
    
    public void run(){
        // 1 Selector thread per Connector.
        if ( privilegedPorts!= null && privilegedPorts.contains(getPort()) ){
            System.out.println
                    ("Creating a SelectorRealtimeThread listening on port "
                    + getPort());
            RealtimeThread rt = new RealtimeThread(){
                public void run(){
                    // This is a workaround for a bug in setting
                    // realtime priorities.
                    PriorityScheduler scheduler = PriorityScheduler.instance();
                    PriorityParameters pp = (PriorityParameters)
                    this.getSchedulingParameters();
                    pp.setPriority(scheduler.getMaxPriority());
                    try{
                        startEndpoint();
                    } catch (Exception ex){
                        logger.log(Level.SEVERE,"selectorThread.errorOnRequest", ex);
                    }
                }
            };
            rt.setName("RealTimeSelectorThread-" + getPort());
            rt.start();
        } else {
            try{
                startEndpoint();
            } catch (Exception ex){
                logger.log(Level.SEVERE,"selectorThread.errorOnRequest", ex);
            }
        }
    }
}

